@@ -1,5 +1,5 @@
 from discord.ext import commands as c
-from discord import FFmpegPCMAudio, ui, Message
+from discord import FFmpegPCMAudio, errors, ui, Message
 from asyncpgw import general
 from typing import Optional
 from asyncio import sleep
@@ -16,7 +16,7 @@ import re
 
 __all__ = ["VoiceTextContext", "get_folder", "voice_purge"]
 
-bot_channels = """bot_channels(
+bot_channels = """bot_voices(
     server bigint,
     vc bigint,
     opt bigint,
@@ -154,18 +154,34 @@ class VoiceTextContext(c.Context):
         self.speakers = tuple(self.speaker.keys())
         self.emotions = tuple(self.emotion.keys())
 
-    async def join(self) -> None:
+    async def connect(self) -> Optional[Message]:
+        """読み上げを開始するVCに接続します。"""
+        try:
+            await self.author.voice.channel.connect()
+        except  errors.ClientException:
+            await self.disconnect()
+        except (TimeoutError, errors.ConnectionClosed):
+            m: Message = await self.send("接続がタイムアウトしました。もう一度コマンドを打ち、再度タイムアウトする場合は時間を置いてコマンドを送信してみてね")
+            return m
+    async def disconnect(self) -> None:
+        "読み上げを開始してるVCから退出します"
+        await self.author.voice.channel.disconnect(force=True)
+        
 
+    async def join(self) -> None:
+        """読み上げを開始するためのデータをDBに保存します。"""
         if not await self.bot_channels.fetch(**self.default_params):
             await self.bot_channels.insert(**self.default_params)
 
-        await self.read_voice.update(tc=self.channel.id, **self.default_params)
+        await self.bot_channels.update(tc=self.channel.id, **self.default_params)
 
-        await self.read_voice.update(join_time=datetime.now(), **self.default_params)
+        await self.bot_channels.update(join_time=datetime.now(), **self.default_params)
+        await self.connect()
 
-
-    async def leave(self):
+    async def leave(self) -> None:
+        "読み上げをしているVCから退出します。"
         await self.bot_channels.update(tc=None, **self.default_params)
+        await self.disconnect()
 
     async def load_data(self) -> dict:
         """ユーザーのデータを取得する
@@ -440,6 +456,7 @@ class VoiceTextContext(c.Context):
         return await self.send(embed=e)
 
     async def save_speaker(self) -> None:
+        "話者を設定する"
         if not (data := await self.user_voice.fetch(member=self.author.id)):
             return await self._error()
 
@@ -449,6 +466,7 @@ class VoiceTextContext(c.Context):
         return await self.send(embed=e, view=Speaker(self.ctx, data["use_number"]))
 
     async def save_emotion(self):
+        "感情を設定する"
         if not (data := await self.user_voice.fetch(member=self.author.id)):
             return await self._error()
 
@@ -458,6 +476,7 @@ class VoiceTextContext(c.Context):
         return await self.send(embed=e, view=Emotion(self.ctx, data["use_number"]))
 
     async def get_number(self, opt):
+        "感情レベル、音程、速度、音量からなんの設定をするか選択し、そのパラメータの値を設定する"
         if opt == "emotion_level":
             limit = range(1, 5)
         elif opt == "pitch":
@@ -477,6 +496,7 @@ class VoiceTextContext(c.Context):
         return m
 
     async def save_number(self, opt, jpn_opt, min: int, max: int):
+        "指定されたオプションを保存する"
         if not (data := await self.user_voice.fetch(member=self.author.id)):
             return await self._error()
 
@@ -489,22 +509,27 @@ class VoiceTextContext(c.Context):
         number = await self.get_number(opt)
 
         params = {
+            opt: number,
             "member": self.author.id,
             "set_number": data["use_number"]
         }
 
-        await self.user_voices.update(emotion_level=number, **params)
+        await self.user_voices.update(**params)
 
     async def save_emotion_level(self):
+        "感情レベルを設定する"
         await self.save_number("emotion_level", "感情レベル", 1, 4)
 
     async def save_pitch(self):
+        "音程を設定する"
         await self.save_number("pitch", "音程", 50, 200)
 
     async def save_speed(self):
+        "速度を設定する"
         await self.save_number("speed", "速度", 50, 400)
 
     async def save_volume(self):
+        "音量を設定する"
         await self.save_number("volume", "音量", 50, 200)
 
     async def show(self) -> dict:
@@ -529,6 +554,7 @@ class VoiceTextContext(c.Context):
         return params
 
 async def voice_purge(bot, member):
+    "読み上げたメッセージの自動削除"
     auto_delete = general.Pg(bot, 'voice_auto_delete')
     bot_channels = general.Pg(bot, 'bot_voices')
 
